import os.path
from flask import Flask
from flask.ext.sqlalchemy import SQLAlchemy
from flask.ext.script import Manager
from flask.ext.migrate import Migrate, MigrateCommand
from flask_login import LoginManager
import logging
from classes import User
import util


app = Flask(__name__)
if os.path.isfile(os.path.join(os.path.dirname(os.path.realpath(__file__)), 'config.py')):
    app.config.from_pyfile('config.py')
else:
    app.config.from_pyfile('/etc/payroll2python.config.py')
app.secret_key = app.config['SECRET_KEY']

db = SQLAlchemy(app)

login_manager = LoginManager()
login_manager.init_app(app)

migrate = Migrate(app, db, directory="payroll2python/migrations")
manager = Manager(app)
manager.add_command('db', MigrateCommand)
from models import *

handler = logging.FileHandler(app.config["LOG_FILENAME"])
app.logger.addHandler(handler)

app.jinja_env.globals.update(daterange=util.daterange)

from views import *

@login_manager.user_loader
def load_user(userid):
    """Loads user object for login.

    Parameters:
      - `userid`: An id for the user (typically a NetID).
    """

    employee_info = client.get_user_by_netid(userid)
    if employee_info:
        return User(userid,
                    fullname=employee_info.fullname,
                    type=employee_info.type,
                    descduty=employee_info.descduty,
                    payrate=employee_info.payrate,
                    account_num=employee_info.account_num,
                    email=employee_info.email)
    return User(userid)


if __name__ == "__main__":
    app.run(debug=True, port=5002)
