from app import db
import models
from classes import Employee, PayPeriod, WorkUnit


def db_add(item):
    db.session.add(item)
    db.session.commit()

def mark_payperiod_complete(start_date, netid, is_complete):
    pp = pay_period_exists(start_date=start_date, netid=netid)
    pp_id = pp.pp_id
    db_pp = models.PayPeriods.query.filter_by(pp_id=pp_id).first()
    
    db_pp.complete = is_complete
    db.session.commit()

def insert_or_update_work_unit(start, end, start_date, user):
    # work = PayPeriods.query.\
    #     join(WorkUnits.pp_id).\
    #     filter_by(start=start, end=end, netid=netid).\
    #     first()
    # Upon calling pp_exists we know it does exist
    pp = pay_period_exists(start_date=start_date, netid=user.netid)

    found = 0

    pp_id = pp.pp_id
    db_work_units = models.WorkUnits.query.filter_by(pp_id=pp_id)

    for unit in db_work_units:
        if unit.start.date() == start.date():
            # Update work unit's start and end times
            found = 1
            unit.start = start
            unit.end = end

    if not found:
        new_work_unit = models.WorkUnits(start=start, end=end, pp_id=pp_id)
        db.session.add(new_work_unit)

    db.session.commit()


def check_to_delete_work_unit(start_date, start_day, user):
    pp_id = pay_period_exists(netid=user.netid, start_date=start_date).pp_id
    db_work_units = models.WorkUnits.query.filter_by(pp_id=pp_id)
    for work_unit in db_work_units:
        if work_unit.start.date() == start_day.date():
            db.session.delete(work_unit)
    db.session.commit()


def make_db_pay_period(start_date, netid):
    return models.PayPeriods(start_date=start_date, emp_id=netid)


def get_payperiod(start_date, netid):
    query = models.PayPeriods.query.filter_by(emp_id=netid, start_date=start_date)
    row = query.first()
    
    if not row:
        return None

    return db_to_pp(row)


def create_new_payperiod(netid, start_date):
    employee = get_employee_by_netid(netid)
    pp = PayPeriod(employee, start_date)
    db_add(pp_to_db(pp))
    return pp


def pay_period_exists(start_date, netid):
    """ Deprecated, use get_payperiod """
    return models.PayPeriods.query.filter_by(emp_id=netid,
                                      start_date=start_date).first()


def pp_to_db(payperiod):
    return models.PayPeriods(start_date=payperiod.start_date,
                      emp_id=payperiod.employee.netid,
                      payrate=payperiod.payrate)


def db_to_pp(db_payperiod):
    db_employee = models.Employees.query.filter_by(netid=db_payperiod.emp_id).first()
    employee = db_to_employee(db_employee)
    db_work_units = models.WorkUnits.query.filter_by(pp_id=db_payperiod.pp_id)
    work_units = []
    for db_work in db_work_units:
        work = db_to_work_unit(db_work)
        work_units.append(work)
    return PayPeriod(employee=employee,
                     start_date=db_payperiod.start_date,
                     work_units=work_units,
                     complete=db_payperiod.complete,
                     payrate=db_payperiod.payrate)


def db_to_work_unit(db_work_unit):
    return WorkUnit(start=db_work_unit.start, end=db_work_unit.end)


def db_to_employee(db_employee):
    return Employee(netid=db_employee.netid,
                    fullname=db_employee.fullname,
                    type=db_employee.type,
                    description=db_employee.descduty,
                    payrate=db_employee.payrate,
                    account_num=db_employee.account_num,
                    email=db_employee.email)


def user_exists(netid):
    # Returns true if netID exists in the database
    return models.Employees.query.filter_by(netid=netid).first()


def user_to_employee(user):
    return Employee(netid=user.netid,
                    fullname=user.fullname,
                    type=user.type,
                    description=user.descduty,
                    payrate=user.payrate,
                    account_num=user.account_num,
                    email=user.email)


def update_employee(emp):
    guy = models.Employees.query.filter_by(netid=emp.netid).first()

    # Update guy's info (we know at this point that the fields are valid)
    if guy.account_num != emp.account_num:
        guy.account_num = emp.account_num
    if guy.fullname != emp.fullname:
        guy.fullname = emp.fullname
    if guy.descduty != emp.descduty:
        guy.descduty = emp.descduty
    if guy.payrate != emp.payrate:
        guy.payrate = emp.payrate
    if guy.email != emp.email:
        guy.email = emp.email
    if guy.type != emp.type:
        guy.type = emp.type

    db.session.commit()


def get_user_by_netid(netid):
    return models.Employees.query.filter_by(netid=netid).first()

def get_employee_by_netid(netid):
    return models.Employees.query.filter_by(netid=netid).first()

def get_user_list():
    user_list = []
    emp_info = models.Employees.query.all()
    for x in emp_info:
        user_list.append(x.netid)
    return user_list


def make_employee(form):
    return models.Employees(netid=form.netid.data,
                     email=form.email.data,
                     fullname=form.name.data,
                     descduty=form.title.data,
                     type=form.employee_type.data,
                     payrate=float(form.hourly_wage.data),
                     account_num=int(form.account_num.data))
