import client
import datetime


CSV_HEADER = "netid,signin,signout,total_break_time,account_code,system_notes\n"
CSV_DATE_FORMAT = "%Y/%m/%d %H:%M"


def get_warnings(start_date):
    warnings = []

    user_list = client.get_user_list()

    # Check for missing payperiods
    for netid in user_list:
        if not client.pay_period_exists(start_date, netid):
            warnings.append(netid + " has no hours recorded for this " +
                "payperiod.")

    # Check for incomplete payperiods
    for netid in user_list:
        pp = client.pay_period_exists(start_date, netid)
        if pp:
            pp = client.db_to_pp(pp)
            if not pp.complete:
                warnings.append(netid + " has not marked this payperiod as " +
                    "complete (generating hours anyway)")

    return warnings


def get_csv(start_date):
    user_list = client.get_user_list()

    data = ""
    data += CSV_HEADER

    for netid in user_list:
        user = client.user_exists(netid)
        pp = client.pay_period_exists(start_date, netid)
        if pp:
            pp = client.db_to_pp(pp)
            for unit in pp.work_units:
                start = datetime.datetime.strftime(unit.start, CSV_DATE_FORMAT)
                end = datetime.datetime.strftime(unit.end, CSV_DATE_FORMAT)
                breaktime = unit.get_break_time()
                account_code = user.account_num
                data += "{},{},{},{},{}\n".format(netid, start, end, breaktime,
                    account_code)
    
    return data
