from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.utils import formatdate
import smtplib

EMAIL_TXT = (
    "Hello! This email has been sent to confirm "
    "that you signed up on the OSS Employee Info Sheet.\n"
    "If this is not you, please ignore this message.\n\n"
    "Your Employee Info Sheet:\n\n"
    "NetID: {0}\n"
    "Name: {1}\n"
    "Employee Type: {2}\n"
    "Hourly Wage: ${3:.2f}\n"
    "Account Number: {4}\n"
    "Title: {5}\n"
)
FROM = "payroll@oss.rutgers.edu"
SUBJECT = "Email Confirmation from OSS Signup"


def send_mail(employee, body="", fro=FROM, subject=SUBJECT):
    """Send a mail given an employee object from
    Models.employee and a body of text
    """
    body = format_text(employee)
    msg = MIMEMultipart()
    msg['From'] = fro
    msg['To'] = employee.email
    msg['Date'] = formatdate(localtime=True)
    msg['Subject'] = subject
    msg.attach(MIMEText(body))

    try:
        smtp = smtplib.SMTP('localhost')
        smtp.sendmail(fro, employee.email, msg.as_string())
        smtp.quit()
    except smtplib.SMTPException:
        print("Error: unable to send email")
        import traceback
        traceback.print_exc()


def format_text(employee, txt=EMAIL_TXT):
    txt.format(
        employee.netid,
        employee.fullname,
        employee.type,
        employee.payrate,
        employee.account_num,
        employee.descduty
    )
    return txt
