from app import db


class Employees(db.Model):
    """Employee object stores all necessary information for a worker."""
    __tablename__ = 'empinfo'
    netid = db.Column(db.String(20), primary_key=True, nullable=False)
    fullname = db.Column(db.String(30))
    type = db.Column(db.Integer, nullable=False)
    descduty = db.Column(db.String(50))
    payrate = db.Column(db.Float(precision=2))
    email = db.Column(db.String(30))
    account_num = db.Column(db.Integer)


class PayPeriods(db.Model):
    """A two week pay period for a worker
    pp_id       : integer   -> primary key for payperiods identification
    start_date  : date      -> the day the pay period was started
    emp_id      : string    -> Fkey: unique netid of worker
    """
    __tablename__ = 'payperiod'
    pp_id = db.Column(db.Integer, primary_key=True, nullable=False)
    start_date = db.Column(db.Date(), nullable=False)
    emp_id = db.Column(db.String(20), db.ForeignKey('empinfo.netid'),
                       nullable=False)
    complete = db.Column(db.Boolean, nullable=False, default=False)
    payrate = db.Column(db.Float(precision=2), nullable=False)
    # work_units = db.relationship('workunit',
    #  backref='payperiod', lazy='dynamic')


class WorkUnits(db.Model):
    """Represents one day of work. Can't store intervals without epochs!
    workid      : integer   -> unique primary key of the work unit
    start_time  : datetime  -> when the work was started
    end_time    : datetime  -> when the work was ended
    """
    __tablename__ = 'workunit'
    work_id = db.Column(db.Integer, primary_key=True, nullable=False)
    pp_id = db.Column(db.Integer, db.ForeignKey('payperiod.pp_id'),
                      nullable=False)
    start = db.Column(db.DateTime, nullable=False)
    end = db.Column(db.DateTime, nullable=False)
