import client, util

ACCOUNTS = [
    {"name": "Enterprise Messaging", "account_num": 202823},
    {"name": "EM Mobile Apps", "account_num": 289243}
]

def __merge_dict(dict1, dict2):
    ret = {}
    
    for key in dict1:
        ret[key] = dict1[key]

    for key in dict2:
        if key in ret:
            ret[key] += dict2[key]
        else:
            ret[key] = dict2[key]

    return ret


def clamp_workunits_by_daterange(payperiod, start_date, end_date):
    new_workunits = []

    for workunit in payperiod.work_units:
        date = workunit.start.date()
        if start_date <= date and end_date >= date:
            new_workunits.append(workunit)

    payperiod.work_units = new_workunits
    return payperiod


def get_payperiods_by_daterange(start_date, end_date, user):
    current = start_date

    ret = []
    
    while current <= end_date:
        pp_start_date = util.get_start_date(current)
        pp_end_date = util.get_end_date(pp_start_date)

        payperiod = client.get_payperiod(pp_start_date, user.netid)
        if payperiod:
            ret.append(clamp_workunits_by_daterange(payperiod, current, end_date))
 
        current = util.next_pp(pp_start_date)

    return ret


def get_statistics_by_user(start_date, end_date, user):
    ret = {"shifts": 0, "hours": 0, "total": 0}

    payperiods = get_payperiods_by_daterange(start_date, end_date, user)

    for payperiod in payperiods:
        ret['shifts'] += payperiod.get_total_shifts()
        ret['hours'] += payperiod.get_total_billable_hours()
        ret['total'] += payperiod.get_total_payroll_amount()

    return ret


def get_users_by_account(account_num):
    ret = []

    netid_list = client.get_user_list()
    for netid in netid_list:
        user = client.get_user_by_netid(netid)
        if user.account_num == account_num:
            ret.append(user)

    return ret

def get_statistics_by_account(start_date, end_date, account):
    """Returns a dict of statistics. Takes a datetime starting date and ending
    date, as well as an account dict with a name key and account_num key.
    Returns a dict in the following format: 
    name => The name of the account
    shifts => The number of shifts logged for this account for the given time 
              period
    hours => The number of hours logged
    total => The total payroll amount, in dollars
    """
    ret = {"name": account['name'], "shifts": 0, "hours": 0, "total": 0}
    
    account_num = account['account_num']

    user_list = get_users_by_account(account_num)
    for user in user_list:
        stats = get_statistics_by_user(start_date, end_date, user)
        ret = __merge_dict(ret, stats)

    return ret


def get_statistics(start_date, end_date):
    """Returns a list of statistics of all accounts given by 
    get_statistics_by_account plus an additional entry for the combined totals.
    """
    ret = []
    combined = {"name": "", "shifts": 0, "hours": 0, "total": 0.0}
        
    for account in ACCOUNTS:
        stats = get_statistics_by_account(start_date, end_date, account)
        combined = __merge_dict(combined, stats)
        combined['name'] = "Combined"
        ret.append(stats)

    ret.append(combined)
    return ret
