"""Timesheet generation"""

# This file is deprecated :(

from wand.drawing import Drawing
from wand.image import Image
import collections
import datetime
from payroll2python.util import daterange

# Location of timesheet png
TIMESHEET_PATH = 'payroll2python/static/img/timesheetclean.png'

# Scaling factor
SF = 64

# Date displacement
DD = 29

# Work unit displacement
WD = 92


class WorkUnit():
    """A unit of work.
    start_time : datetime  -> when the work was started
    length     : timedelta -> how long the work took
    """
    def __init__(self, start_time, length):
        self.start_time = start_time
        self.length = length

    def end_time(self):
        """Time a WorkUnit ends"""
        return self.start_time + datetime.timedelta(hours=self.length)

    def weekday(self):
        """Number day of the week starting on Saturday = 0"""
        return (self.start_time.weekday() + 2) % 7


class PayPeriod():
    """A two week pay period
    employee   : Employee   -> who the work was done by
    start_date : date       -> the first day of the pay period
    work_units : [WorkUnit] -> all the work done in a week
    """
    def __init__(self, employee, start_date, work_units):
        self.employee = employee
        self.start_date = start_date
        self.work_units = work_units

    def end_date(self):
        """Date that a pay period ends (one day before 2 weeks)"""
        return self.start_date + datetime.timedelta(weeks=1, days=6)

    def next_pay_period(self):
        """Date of next pay period (2 weeks)"""
        return self.start_date + datetime.timedelta(weeks=2)

    def week_totals(self):
        """A tuple containing the total hours worked for each week"""
        total_week1 = 0
        total_week2 = 0
        for work_unit in self.work_units:
            if is_first_week(self.start_date, work_unit.start_time.date()):
                total_week1 += work_unit.length
            else:
                total_week2 += work_unit.length
        return total_week1, total_week2


# An employee
# name   : string -> the name of the employee
# duties : string -> their job description
# rate   : float  -> their hourly rate
Employee = collections.namedtuple('Employee', 'name duties rate type')


def is_first_week(start_date, checked_date):
    """Determine which week in the pay period a date is"""
    return checked_date < (start_date + datetime.timedelta(weeks=1))


def draw_dates(draw, pay_period):
    """Draw the dates in a PayPeriod (2 weeks of dates starting on Saturday"""
    for date in daterange(pay_period.start_date, pay_period.next_pay_period()):
        if is_first_week(pay_period.start_date, date):
            draw.text(DD + (((date.weekday() + 2) % 7) + 1)*SF,
                      128, date.strftime("%m  %d"))
        else:
            draw.text(DD + (((date.weekday() + 2) % 7) + 1)*SF,
                      230, date.strftime("%m  %d"))


def draw_work_units(draw, work_units, start_date):
    """Draw all the units of work done in a Pay Period"""
    for work_unit in work_units:
        if is_first_week(start_date, work_unit.start_time.date()):
            draw.text(WD + work_unit.weekday()*SF,
                      148, work_unit.start_time.strftime("%I  %M"))
            draw.text(WD + work_unit.weekday()*SF,
                      170, work_unit.end_time().strftime("%I  %M"))
            draw.text(WD + work_unit.weekday()*SF,
                      190, str(work_unit.length))
        else:
            draw.text(WD + work_unit.weekday()*SF,
                      250, work_unit.start_time.strftime("%I  %M"))
            draw.text(WD + work_unit.weekday()*SF,
                      270, work_unit.end_time().strftime("%I  %M"))
            draw.text(WD + work_unit.weekday()*SF,
                      290, str(work_unit.length))


def draw_employee_data(draw, pay_period):
    """Draws all information about an employee (along with
    when the Pay Period starts)
    """
    employee = pay_period.employee
    start_date = pay_period.start_date
    total_week1, total_week2 = pay_period.week_totals()
    draw.text(60, 68, employee.name)
    draw.text(550, 68, employee.duties)
    draw.text(740, 42, employee.type)
    draw.text(690, 145, str(employee.rate))
    draw.text(656, 106, "NBCS")
    draw.text(610, 40, start_date.strftime("%m-%d-%y"))
    draw.text(540, 190, str(total_week1))
    draw.text(540, 290, str(total_week1))
    draw.text(710, 166, str(total_week1 + total_week2))
    draw.text(710, 194, str((total_week1 + total_week2) * employee.rate))


def generate_paysheet(pay_period):
    """Create a paysheet, draws info from the
    pay period to an image at TIMESHEET_PATH
    """
    with Drawing() as draw:
        with Image(filename=TIMESHEET_PATH) as image:
            draw_dates(draw, pay_period)
            draw_work_units(draw, pay_period.work_units, pay_period.start_date)
            draw_employee_data(draw, pay_period)
            draw(image)
            image.save(filename='ayylmao.png')
