"""
Payroll3Python

User object used for authentication.
"""
from wtforms import TextField, PasswordField, validators
from app import app
import client, mail
from classes import User

def get_user(fields):
    """Gets a user object from fields.

    :Returns:
      A User object from the username extracted from the argument.
    """
    return User(fields['username'])


def update_user(employee_form):
    employee = client.make_employee(employee_form)

    if client.user_exists(employee_form.netid.data):  # NetID exists in database
        client.update_employee(employee)
        app.logger.info("INFO:\t{0}[{1}] updated user information"
                        " in the database."
                        .format(employee.fullname, employee.netid))
    else:  # Adds employee to database
        client.db_add(employee)
        app.logger.info("INFO:\tSuccessful user signup: {0}[{1}]."
                            .format(employee.fullname, employee.netid))

        # Send an email with the send_mail() method
        try:
            mail.format_text(employee)
            mail.send_mail(employee)
            flash("Success!")
            flash("Email confirmation sent to {0}."
                      .format(employee.email))
            app.logger.info("INFO:\tEmail confirmation sent to {0}."
                                .format(employee.email))
        except:
            app.logger.error("ERROR:\tproblem occurred while"
                                 " trying to send mail.")
            flash("The employee was successfully created, but the email confirmation failed to send to {0}."
                      .format(employee.email))

