import collections
import datetime

EPOCH = datetime.date(2014, 9, 27)
PP_FORMAT = "%B %d, %Y"
DAY_FORMAT = "%Y-%m-%d"
PayPeriod = collections.namedtuple('PayPeriod', 'start_date end_date')


def daterange(start_date, end_date):
    ret = []
    for n in range(int((end_date - start_date).days) + 1):
        ret.append(start_date + datetime.timedelta(n))
    return ret


def previous_pp(date, i):
    """Gets the previous pay period in proper format from a date object"""
    return date - datetime.timedelta(weeks=2*i)

def next_pp(start_date):
    return start_date + datetime.timedelta(weeks=2)


def get_start_date(start_date=None):
    if start_date is None:
        start_date = datetime.date.today()
    else:
        if type(start_date) is str:
            start_date = datetime.datetime.strptime(
                    start_date, "%m-%d-%Y").date()

    if (int)((start_date - EPOCH).days/7) % 2:
        start_date = start_date - datetime.timedelta(weeks=1)

    # Set start_date to start of the week (saturday) if it is not already
    while start_date.weekday() != 5:
        start_date = start_date - datetime.timedelta(days=1)

    return start_date


def get_end_date(start_date):
    # Set the end_date to the day before the next pay period begins
    return start_date + datetime.timedelta(weeks=1, days=6)


def get_weeks(start_date, end_date):
    # List of the two weeks available
    weeks = []
    # Range of a week for the two weeks in the pay period.
    weeks.append(daterange(start_date,
                 start_date + datetime.timedelta(days=6)))
    weeks.append(daterange(start_date + datetime.timedelta(weeks=1),
                 end_date))
    return weeks


def get_pay_periods(start_date, end_date):
    # Add the current pay period to the array (properly formatted)
    pay_periods = [PayPeriod(start_date=start_date.strftime(PP_FORMAT),
                             end_date=end_date.strftime(PP_FORMAT))]

    # Add an additional 4 pay periods to the array
    for i in range(1, 5):
        pay_periods.append(PayPeriod(start_date=previous_pp(start_date, i),
                                     end_date=previous_pp(end_date, i)))
    return pay_periods


def data_to_datetime(date, time):
    """Combines date object and time string (e.g. 1300) into datetime object"""
    time = datetime.datetime.strptime(time, '%H%M').time()
    return datetime.datetime.combine(date, time)
